/**************************************************************************//**
*   @file   mb_interrupts.c
*   @brief  Interrupts handling functions driver file.
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: $WCREV$
******************************************************************************/




#include <stdio.h>
#include "mb_interrupts.h"

extern volatile char rxData;

/******************************************************************************
* @brief UART Interrupt Handler.
*
* @param None.
*
* @return None.
******************************************************************************/
void uartIntHandler(void)
{
	// If UART has triggered a RX DATA Interrupt
	// store the read value in rxData
	if(Xil_In32(UART_BASEADDR + 0x08) & 0x01)
	{
		// Save read data from UART
		rxData = Xil_In32(UART_BASEADDR);
	}
}

/******************************************************************************
* @brief GPIO Interrupt Handler.
*
* @param None.
*
* @return None.
******************************************************************************/
void gpioIntHandler(void)
{
	char intSource = 0x00;

	// If GPIO Triggered an Interrupt
	if((Xil_In32(GPIO_BASEADDR + 0x120) & 0x01) == 0x01)
	{
		// Check which pin triggered an Interrupt
		intSource = Xil_In32(GPIO_BASEADDR);

		// If both Pins, turn on LED1 and LED2
		if((intSource & 0x03) == 0x03)
		{
			// Turn on LED1 to LED8
			Xil_Out32(LED_BASEADDR, 0xFF);
			// Start Timer
			Xil_Out32(TIMER_BASEADDR + 0x00, 0x000001C0);
		}
		// If Pin1, turn on LED5 to LED8
		else if((intSource & 0x01) == 0x01)
		{
			// Turn on LED5 to LED8
			Xil_Out32(LED_BASEADDR, 0xF0);
			// Start Timer
			Xil_Out32(TIMER_BASEADDR + 0x00, 0x000001C0);
		}
		// If Pin2, turn on LED1 to LED4
		else if((intSource & 0x02) == 0x02)
		{
			// Turn on LED1 to LED4
			Xil_Out32(LED_BASEADDR, 0x0F);
			// Start Timer
			Xil_Out32(TIMER_BASEADDR + 0x00, 0x000001C0);
		}
	}

	// Clear GPIO Interrupts
	Xil_Out32(GPIO_BASEADDR + 0x120, 0x01);
}

/******************************************************************************
* @brief Timer Interrupt Handler.
*
* @param None.
*
* @return None.
******************************************************************************/
void timerIntHandler(void)
{
	// Write Load Register with value for 1 second count
	Xil_Out32(TIMER_BASEADDR + 0x04, 0xFA0A1F01);
	// Load the Count Register
	Xil_Out32(TIMER_BASEADDR + 0x00, 0x00000060);
	// Turn OFF LEDs
	Xil_Out32(LED_BASEADDR, 0x00);

	// Clear Timer Interrupts
	Xil_Out32(TIMER_BASEADDR + 0x00, 0x00000100);
}

/******************************************************************************
* @brief Enable Interrupts.
*
* @param None.
*
* @return None.
******************************************************************************/
void enableInterrupts(void)
{
	// Register UART Interrupt Service Routine
	XIntc_RegisterHandler(INTC_BASEADDR, UART_INTERRUPT, (XInterruptHandler)uartIntHandler, (void *) 0);

	// Register GPIO Interrupt Service Routine
	XIntc_RegisterHandler(INTC_BASEADDR, GPIO_INTERRUPT, (XInterruptHandler)gpioIntHandler, (void *) 0);

	// Register TIMER0 Interrupt Service Routine
	XIntc_RegisterHandler(INTC_BASEADDR, TIMER_INTERRUPT, (XInterruptHandler)timerIntHandler, (void *) 0);

	// Enable Interrupt Controller
	XIntc_MasterEnable(INTC_BASEADDR);
	XIntc_EnableIntr(INTC_BASEADDR, UART_INTERRUPT_MASK | GPIO_INTERRUPT_MASK | TIMER_INTERRUPT_MASK);

	// Enable UART Interrupts
	Xil_Out32(UART_BASEADDR + 0x0C, (1 << 4));

	// Set GPIO as Inputs
	Xil_Out32(GPIO_BASEADDR + 0x04, 0x03);

	// Set LEDS as Outputs
	Xil_Out32(LED_BASEADDR, 0x00);

	// Enable GPIO Interrupts
	Xil_Out32(GPIO_BASEADDR + 0x11C, 0xFFFFFFFF);
	Xil_Out32(GPIO_BASEADDR + 0x128, 0x01);

	// Enable TIMER Interrupts and Initial Settings
	Xil_Out32(TIMER_BASEADDR + 0x04, 0xFA0A1F01);
	Xil_Out32(TIMER_BASEADDR + 0x00, 0x00000060);

	// Enable MicroBlaze Interrupts
	microblaze_enable_interrupts();
}
